#Code by Uyen Nguyen to accompany the 2021 Bridges paper "Folding Functions II: Methods for Mathematically Manipulating Miura-ori Models"
#Many thanks to Marcus Michelen for assistance.

import math
from dxfwrite import DXFEngine as dxf
from scipy.optimize import fsolve

##User Inputs
##Input your function below
def f(x):
    return .3*(x**3 + 3*x**2 + x + 2)

##Output Filename
file_name = "1312.dxf"

## Start of Interval
t_first = -3

##End of Interval
t_last = 1

##Size of first step
step1 = .5

##Extra amount of space on top and bottom of curve
extra_y_spacing = 2

##Initial Angle of zigzag  in radians
angle = 1.25

##Number of zigzags - even recommended
numzig = 2

##Space between zigzags
zagspace = .5

## Amount to scale the output drawing by
scale = 3

##END OF INPUTS##

##Mesh points
M = 10000
step = (t_last - t_first)/M

## Define the mesh of points on x-axis
Xmesh = list(range(M+1))
for j in Xmesh:
    Xmesh[j] = t_first+ step*j

## Define the mesh of points y-axis
Ymesh = list(range(M+1))
for j in Ymesh:
    Ymesh[j] = f(Xmesh[j])

##Compute the border
y_top = max(Ymesh) + extra_y_spacing + (numzig - 1)*zagspace
y_bottom = min(Ymesh) - extra_y_spacing

## Initialize the Drawing
drawing = dxf.drawing(file_name)

## Draw the border
drawing.add(dxf.line( (scale*t_first,scale*y_bottom), (scale*t_first,scale*y_top), color=7 ))
drawing.add(dxf.line( (scale*t_first,scale*y_top), (scale*t_last,scale*y_top), color=7 ))
drawing.add(dxf.line( (scale*t_last,scale*y_bottom), (scale*t_last,scale*y_top), color=7 ))
drawing.add(dxf.line( (scale*t_first,scale*y_bottom), (scale*t_last,scale*y_bottom), color=7 ))

##Draw the first non-border vertical line
drawing.add(dxf.line( (scale*(t_first+step1),scale*y_bottom), (scale*(t_first+step1),scale*y_top), color=4 ))

##Place the first zig, it's centered at f(t_first + step1/2) and has the defined angle
##terminates at the border and 1st vertical line, need to solve for endpoints

slope = math.tan(angle)
x1 = t_first
x2 = t_first + step1
y1 = f(t_first + step1/2) - slope*step1/2
y2 = f(t_first + step1/2) + slope*step1/2

#repeat for the number of zigzags
for j in (list(range(numzig))):
    xi=x1
    yi= y1+ j*zagspace
    xf = x2
    yf= y2 + j*zagspace
    drawing.add(dxf.line( (scale*xi,scale*yi), (scale*xf,scale*yf), color=2 ))
    

#draw the rest of the crease pattern
#reflection line
def rline(x1, y1, x, i):
    return (x - x1)*slope*(-1)**i+y1

i = 1
x_old = x2
y_old = y2
while x_old < t_last:
    x_mid = fsolve( lambda x: f(x) - rline(x_old, y_old, x, i), x_old)
    diff = x_mid - x_old
    x_new = x_old + 2*diff
    y_new = rline(x_old, y_old, x_new, i)
    drawing.add(dxf.line( (scale*x_new,scale*y_bottom), (scale*x_new,scale*y_top), color=4 ))
    for j in (list(range(numzig))):
        xi = x_old
        yi = y_old + j*zagspace
        xf = x_new
        yf = y_new + j*zagspace
        drawing.add(dxf.line( (scale*xi,scale*yi), (scale*xf,scale*yf), color=2 ))
    i +=1
    x_old = x_new
    y_old = y_new
    

drawing.save()
